
dnl ----------------------------------------------------------------------------
dnl General libs
dnl ----------------------------------------------------------------------------

dnl
dnl Checks for a valid libjpeg
dnl Parameters: Optional search directory
dnl Defines:    HAVE_LIBJPEG
dnl Vars:       FOUND_LIBJPEG, LIBJPEG_CFLAGS, LIBJPEG_LIBS
dnl
dnl @synopsis AC_CHECK_LIBJPEG()
dnl
AC_DEFUN([AC_CHECK_LIBJPEG], [

	USER_DIR=$1
	SAVED_CFLAGS=$CFLAGS
	CFLAGS=
	LIBJPEG_CFLAGS=
	LIBJPEG_LIBS=

	AC_MSG_CHECKING([for libjpeg])

	PREFIX="$USER_DIR /usr /usr/local /sw /opt"
	for dir in $PREFIX ; do		
		CFLAGS="-I$dir/include -L$dir/lib $CFLAGS"
		AC_TRY_COMPILE([  #include <stdio.h>
				  #include <jpeglib.h>],
				[ struct jpeg_compress_struct cinfo;
				  struct jpeg_error_mgr jerr;
				  jpeg_set_defaults (&cinfo);],
					
				[ FOUND_LIBJPEG=yes
				  LIBJPEG_LIBS="-L$dir/lib -ljpeg"
				  LIBJPEG_CFLAGS="-I$dir/include"
				  break],
				  
				[ FOUND_LIBJPEG=no ])
		CFLAGS=$SAVED_CFLAGS
	done
	CFLAGS=$SAVED_CFLAGS

	if test $FOUND_LIBJPEG = yes ; then
		AC_MSG_RESULT([yes])
		AC_DEFINE(HAVE_LIBJPEG)
	else
		AC_MSG_RESULT([no.])
	fi
])


dnl
dnl Checks for a valid libavcodec
dnl Parameters: Optional search directory
dnl Defines:    HAVE_LIBAVCODEC
dnl Vars:       FOUND_LIBAVCODEC, LIBAVCODEC_CFLAGS, LIBAVCODEC_LIBS
dnl
dnl @synopsis AC_CHECK_LIBAVCODEC()
dnl
AC_DEFUN([AC_CHECK_LIBAVCODEC], [

	USER_DIR=$1
	SAVED_CFLAGS=$CFLAGS
	CFLAGS=
	LIBAVCODEC_CFLAGS=
	LIBAVCODEC_LIBS=

	AC_MSG_CHECKING([for libavcodec (ffmpeg)])

	PREFIX="$USER_DIR /usr /usr/local /sw /opt"
	for dir in $PREFIX ; do		
		CFLAGS="-I$dir/include -L$dir/lib $CFLAGS"
		AC_TRY_COMPILE([#include <ffmpeg/avcodec.h>],
				[ avcodec_register_all();],
					
				[ FOUND_LIBAVCODEC=yes
				  LIBAVCODEC_LIBS="-L$dir/lib -lavcodec"
				  LIBAVCODEC_CFLAGS="-I$dir/include"
				  break],
				  
				[ FOUND_LIBAVCODEC=no ])
		CFLAGS=$SAVED_CFLAGS
	done
	CFLAGS=$SAVED_CFLAGS

	if test $FOUND_LIBAVCODEC = yes ; then
		AC_MSG_RESULT([yes])
		AC_DEFINE(HAVE_LIBAVCODEC)
	else
		AC_MSG_RESULT([no.])
	fi
])


dnl
dnl Checks for a valid libSDL
dnl Parameters: Optional search directory
dnl Defines:    HAVE_LIBSDL
dnl Vars:       FOUND_SDL, SDL_LIBS, SDL_CFLAGS
dnl
dnl @synopsis AC_CHECK_LIBSDL()
dnl
AC_DEFUN([AC_CHECK_LIBSDL], [

	FOUND_SDL=no
	SDL_LIBS=
	SDL_CFLAGS=

	AC_CHECK_PROG(SDL_CONFIG, sdl-config, yes)
	if test "$SDL_CONFIG" ; then
		SDL_LIBS="`sdl-config --libs`"
		SDL_CFLAGS="`sdl-config --cflags`"
		old_incs="$INCS"
		INCS="$INCS $SDL_CFLAGS"
		AC_CHECK_LIB(SDL, SDL_Init, [
			AC_CHECK_HEADERS(SDL/SDL.h, 
				[ FOUND_SDL=yes ],
				[ AC_MSG_WARN(SDL found but headers are missing!!)
					FOUND_SDL=no
				])],
			[FOUND_SDL=no],
			$SDL_LIBS)
		INCS="$old_incs"
	else
		# The initial test failed, but the FreeBSD port of SDL is installed
		# in a strange way... look for the FreeBSD installation. [csp]
		AC_CHECK_PROG(SDL11_CONFIG, sdl11-config, yes)
		if test "$SDL11_CONFIG" ; then
			SDL_LIBS="`sdl11-config --libs`"
			SDL_CFLAGS="`sdl11-config --cflags`"
			old_incs="$INCS"
			INCS="$INCS $SDL_CFLAGS"
			AC_CHECK_LIB(SDL-1.1, SDL_Init, [
				AC_CHECK_HEADERS(SDL11/SDL.h, 
					[ FOUND_SDL=yes ],
					[ AC_MSG_WARN(SDL found but headers are missing!!)
						FOUND_SDL=no
					])],
				[FOUND_SDL=no],
				$SDL_LIBS)
			INCS="$old_incs"
		else
			FOUND_SDL=no
		fi
	fi
	
	if test "$FOUND_SDL" = "yes" ; then
		AC_DEFINE(HAVE_LIBSDL)
	fi
])


dnl
dnl Checks for a valid OpenGL installation
dnl Defines: HAVE_GL (,HAVE_APPLE_GL)
dnl Vars:    FOUND_GL, GL_LIBS, GL_CFLAGS
dnl
dnl @synopsis AC_CHECK_GL()
dnl
AC_DEFUN([AC_CHECK_GL], [

	GL_CFLAGS=
	GL_LIBS=
	SAVED_CFLAGS=$CFLAGS
	CFLAGS=

	case "$host_os" in
		*darwin*)
			CFLAGS="-framework GLUT -framework OpenGL $CFLAGS"
			;;
		*)
			CFLAGS="-lglut -lGLU -lgl $CFLAGS"
			;;
	esac

	AC_CACHE_CHECK(for OpenGL framework, FOUND_GL, [
		AC_TRY_COMPILE(
			[#include <GLUT/glut.h>
			 #include <OpenGL/gl.h>
			 #include <OpenGL/glext.h>],
			 return 0,
			[FOUND_GL=yes],
			[FOUND_GL=no])
	])
	CFLAGS=$SAVED_CFLAGS

	if test $FOUND_GL = yes ; then
		AC_DEFINE(HAVE_GL)
		
		case "$host_os" in
			*darwin*)
				AC_DEFINE(HAVE_APPLE_GL)
				GL_CFLAGS=
				GL_LIBS="-framework GLUT -framework OpenGL"
				;;
			*)
				GL_CFLAGS=
				GL_LIBS="-lglut -lGLU -lgl"
				;;
		esac
	fi
])

dnl ----------------------------------------------------------------------------
dnl Mac OS X checks
dnl ----------------------------------------------------------------------------

dnl Check if there is a valid Carbon development platform
dnl Vars:    FOUND_CARBON, CARBON_CFLAGS, CARBON_LIBS
dnl Defines: HAVE_CARBON
dnl
dnl @synopsis AC_CHECK_CARBON()
dnl
AC_DEFUN([AC_CHECK_CARBON], [

	CARBON_CFLAGS=
	CARBON_LIBS=
	SAVED_CFLAGS=$CFLAGS
	CFLAGS=
	
	AC_CACHE_CHECK(for MacOS X Carbon framework, FOUND_CARBON, [
		CFLAGS="-framework Carbon $CFLAGS"
		AC_TRY_COMPILE(
			[#include <Carbon/Carbon.h>
			 #include <QuickTime/QuickTime.h>],
			return 0,
			[FOUND_CARBON=yes],
			[FOUND_CARBON=no])
	])
	CFLAGS=$SAVED_CFLAGS
	
	if test "$FOUND_CARBON" = "yes" ; then
		CARBON_CFLAGS=
		CARBON_LIBS="-framework QuickTime -framework Carbon -framework AppKit"
									
		AC_DEFINE(HAVE_CARBON)
	fi	
])


dnl Check if there is a valid Cocoa development platform
dnl Vars: FOUND_COCOA
dnl Defines: HAVE_COCOA
dnl
dnl @synopsis AC_CHECK_COCOA()
dnl
AC_DEFUN([AC_CHECK_COCOA], [

	COCOA_CFLAGS=
	COCOA_LIBS="-framework QuickTime -framework Carbon"

	SAVED_CFLAGS=$CFLAGS
	CFLAGS=""
	
	# Try to compile a Cocoa Objective-C program
	#	
	AC_CACHE_CHECK(for MacOS X Cocoa framework, FOUND_COCOA, [
		CFLAGS="-ObjC -framework Cocoa $CFLAGS"
		AC_TRY_COMPILE(
			[#include <Cocoa/Cocoa.h>],
			return 0,
			[FOUND_COCOA=yes],
			[FOUND_COCOA=no])
	])
	CFLAGS=$SAVED_CFLAGS
	
	if test "$FOUND_COCOA" = "yes" ; then
		COCOA_CFLAGS=
		COCOA_LIBS="-framework Cocoa"

		AC_DEFINE(HAVE_COCOA)
	fi
])


dnl ----------------------------------------------------------------------------
dnl Tcl/Tk
dnl ----------------------------------------------------------------------------

dnl
dnl @synopsis AC_CHECK_TCL([TCL_DIR],[TCL_VERSION],[TCL_VERSION_ALT])
dnl

AC_DEFUN([AC_CHECK_TCL], [

	TARGET_TCL_DIR=$1
	TCLTK_VERSION=$2
	TCLTK_VERSION_ALT=$3

	FOUND_TCL_INC=no
	FOUND_TCL_LIB=no
	FOUND_TCL=no
	
	TCL_INC=
	TCL_LIB=

	if test -d $TARGET_TCL_DIR ; then
		for i in $TARGET_TCL_DIR/generic $TARGET_TCL_DIR/include/tcl${TCLTK_VERSION} $TARGET_TCL_DIR/include $TARGET_TCL_DIR
		do
			if test -d $i ; then
				AC_CHECK_HEADER($i/tcl.h,
						FOUND_TCL_INC=yes,
						FOUND_TCL_INC=no)

				if test "$FOUND_TCL_INC" = "yes"
				then
					TCL_INC=-I$i
					break
				fi
			fi
		done
	else
		AC_MSG_WARN([Tcl not found in $TARGET_TCL_DIR.])
	fi
	
	if test "$FOUND_TCL_INC" = "yes"
	then
		SAVED_LIBS=$LIBS
		
		# Guess most probable tcl lib location
		for i in $TARGET_TCL_DIR/unix $TARGET_TCL_DIR/lib $TARGET_TCL_DIR
		do
			if test -d $i -a "$FOUND_TCL" = no
			then
				LIBS=$SAVED_LIBS
				
				AC_SEARCH_LIBS(Tcl_Init, tcl${TCLTK_VERSION} tcl${TCLTK_VERSION_ALT}, 
				[
					FOUND_TCL=yes

					# If present, we prefer the static version of the library 
					AC_SEARCH_LIBS(Tcl_Init, $i/libtcl${TCLTK_VERSION}.a $i/libtcl${TCLTK_VERSION_ALT}.a, 
					[
					if test -r $i/libtcl${TCLTK_VERSION}.a ; then
						TCL_LIB="$i/libtcl${TCLTK_VERSION}.a"
					else
						TCL_LIB="$i/libtcl${TCLTK_VERSION_ALT}.a"
					fi
					],
					[TCL_LIB="-L$i -ltcl"], 
					-L$i $SAVED_LIBS -lm)
				],
				[], 
				[-L$i $SAVED_LIBS -lm])
			fi
		done

		LIBS=$SAVED_LIBS
	fi
])

dnl --------------------------------------------------------------------------
dnl @synopsis AC_CHECK_TK([TK_DIR],[TK_VERSION],[TK_VERSION_ALT])
dnl
dnl
AC_DEFUN([AC_CHECK_TK], [

	TARGET_TK_DIR=$1
	TCLTK_VERSION=$2
	TCLTK_VERSION_ALT=$3

	FOUND_TK_INC=no
	FOUND_TK_LIB=no
	FOUND_TK=no
	
	TK_INC=
	TK_LIB=

	# Depending on config expect tk.h to be tk source dir or include path
	# AC_CHECK_HEADER(S) are totally inadequate since it is necessary to
	# include tcl.h, Xlib.h, Xutil.h before tk.h.
	
	if test -d $TARGET_TK_DIR ; then	
		for i in $TARGET_TK_DIR/generic $TARGET_TK_DIR/include/tk${TCLTK_VERSION} $TARGET_TK_DIR/include $TARGET_TK_DIR
		do
			AC_MSG_CHECKING(for tk.h in $i)
			if test -r $i/tk.h ; then
				TK_INC=-I$i
				FOUND_TK_INC=yes
				AC_MSG_RESULT(yes)
				break
			fi
			AC_MSG_RESULT(no)
		done
	else
		AC_MSG_WARN([Tk not found in TARGET_TK_DIR.])
	fi
	
	
	if test "$FOUND_TK_INC" = "yes"
	then
		TK_LIB_DIR=
		
		# Guess most probable tk lib location	
		for i in $TARGET_TK_DIR/unix $TARGET_TK_DIR/lib $TARGET_TK_DIR
		do
			AC_MSG_CHECKING(for the Tk library in $i)			
		        if test -d $i ; then
		                TK_LIB_DIR=$i
		                AC_MSG_RESULT(yes)
		                break
		        fi
		done
		
		if test "$TK_LIB_DIR" != ""
		then
			SAVED_LIBS=$LIBS
			LIBS=""
			
			AC_SEARCH_LIBS(Tk_Init, tk${TCLTK_VERSION} tk${TCLTK_VERSION_ALT},
				[ 
		    	    		FOUND_TK_LIB=yes
				],
		        	AC_MSG_RESULT(no),
				-L${TK_LIB_DIR} ${TCL_LIB} $X_LIBS $X_PRE_LIBS  -lXext -lX11 $X_EXTRA_LIBS ${SAVED_LIBS} -lm)
		
			if test -f ${TK_LIB_DIR}/libtk${TCLTK_VERSION}.a ; then
				TK_LIB="${TK_LIB_DIR}/libtk${TCLTK_VERSION}.a"
			else
				if test -f ${TK_LIB_DIR}/libtk${TCLTK_VERSION_ALT}.a ; then
					TK_LIB="${TK_LIB_DIR}/libtk${TCLTK_VERSION_ALT}.a"
				else 
					TK_LIB="-L${TK_LIB_DIR} $LIBS"
				fi
			fi		
			LIBS=$SAVED_LIBS
			
			if test "$FOUND_TK_LIB" = "yes" ; then
				FOUND_TK=yes
			fi
		fi
	fi
])


dnl ----------------------------------------------------------------------------
dnl Compiler options
dnl ----------------------------------------------------------------------------

dnl
dnl AC_CC_OPTION
dnl -------------------------------
dnl Check if GCC supports the option provided and add
dnl it to CFLAGS if available
dnl

AC_DEFUN([AC_CC_OPTION],[
	OPTION=$1
	
	ac_save_CFLAGS=$CFLAGS

	AC_MSG_CHECKING([if $CC supports $OPTION])
	
	if test "$GCC" = yes; then
		CFLAGS="$ac_save_CFLAGS $OPTION"
		_AC_COMPILE_IFELSE([AC_LANG_PROGRAM()],
			[AC_MSG_RESULT([yes])],
			[AC_MSG_RESULT([no])
			 CFLAGS=$ac_save_CFLAGS])
	fi

])


dnl
dnl AC_CC_PCH
dnl -------------------------------
dnl Check if GCC supports precompiled headers and add it to CFLAGS if available.
dnl It needs the name of the header file to precompile.
dnl

AC_DEFUN([AC_CC_PCH],[

	
	ac_save_CFLAGS=$CFLAGS

	AC_MSG_CHECKING([if $CC supports precompiled headers])
	
	if test "$GCC" = yes; then
		
		CFLAGS="$ac_save_CFLAGS -fpch-preprocess -Winvalid-pch"
		
		_AC_COMPILE_IFELSE([AC_LANG_PROGRAM()],
			[AC_MSG_RESULT([yes])
			 PCH_H=$1
			 PCH_GCH=$PCH_H.gch],
			[AC_MSG_RESULT([no])
			 PCH_H=
			 PCH_GCH=
			 CFLAGS=$ac_save_CFLAGS])
	fi

])
