/*
 * FILE:   environment.h
 * AUTHOR: Alvaro Saurin <saurin@dcs.gla.ac.uk>
 *
 * Copyright (c) 2005 University of Glasgow
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Revision: 1.16 $
 * $Date: 2006/02/06 13:34:58 $
 *
 */

#ifndef _ENVIRONMENT_H_
#define _ENVIRONMENT_H_

/*
 * The running environment.
 */
struct run_env {
	const char	*addr;		/* Network address */
        const char	*display;	/* Display subsytem name */
	const char	*display_args;	/* Optional arguments for the display */
        const char	*capture;	/* Capture device name */
	const char	*capture_args;	/* Optional arguments for the capture */ 
	unsigned int	 capture_width;
	unsigned int	 capture_height;
        unsigned int	 mtu;		/* MTU */
        unsigned int	 fps;		/* Frames per second */
        unsigned int	 rx_port;	/* Reception port number */
        unsigned int	 tx_port;	/* Transmission port number */
	const char	*codec;		/* Codec to use for transmission */
	const char	*trace_file;	/* Trace file name */
	const char	*trace_events;	/* Trace events list */
	int		 trace_t_abs;	/* Trace times are absolute */
	const char	*cc_algo;	/* CC system */
};

/*
 * Modifies the given running environment, setting some defaults.
 */
struct run_env* env_init(void);

/*
 * Modifies the given running environment, setting the values 
 * specified with the command line options.
 */ 
int env_load_cmdline(int*, char**, struct run_env*);

/*
 * Destroys a run_env
 */
void env_done(struct run_env*);

#endif 
