/*
 * FILE:     rtp_db.h
 * AUTHOR:   Colin Perkins   <csp@csperkins.org>
 * MODIFIED: Orion Hodson    <o.hodson@cs.ucl.ac.uk>
 *           Markus Germeier <mager@tzi.de>
 *           Bill Fenner     <fenner@research.att.com>
 *           Timur Friedman  <timur@research.att.com>
 *           Ladan Gharai    <ladan@isi.edu>
 *           Steve Smith     <ssmith@vislab.usyd.edu.au>
 *
 * The routines in this file implement the Real-time Transport Protocol,
 * RTP, as specified in RFC 3550. Portions of the code are derived from 
 * the algorithms published in that specification.
 *
 * Copyright (c) 1998-2001 University College London
 * Copyright (c) 2001-2004 University of Southern California
 * Copyright (c) 2003-2005 University of Glasgow
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted provided that the following conditions 
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Computer Science
 *      Department at University College London and by the University of
 *      Southern California Information Sciences Institute.
 * 4. Neither the name of the University nor of the Department may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Revision: 1.2 $
 * $Date: 2005/09/23 15:44:05 $
 *
 * Derived from revision 1.305 of rtp.c
 * 
 */

struct rtp_db;

typedef struct _source {
	struct _source	*next;
	struct _source	*prev;
	uint32_t	 ssrc;
	char		*sdes_cname;
	char		*sdes_name;
	char		*sdes_email;
	char		*sdes_phone;
	char		*sdes_loc;
	char		*sdes_tool;
	char		*sdes_note;
	char		*sdes_priv;
	rtcp_sr		*sr;
	uint32_t	 last_sr_sec;
	uint32_t	 last_sr_frac;
	struct timeval	 last_active;
	int		 sender;
	int		 got_bye;		/* TRUE if we've received an RTCP bye from this source */
	uint32_t	 base_seq;
	uint16_t	 max_seq;
	uint32_t	 bad_seq;
	uint32_t	 cycles;
	int		 received;
	int		 received_prior;
	int		 expected_prior;
	int		 probation;
	uint32_t	 jitter;
	uint32_t	 transit;
	uint32_t	 magic;			/* For debugging... */
} rtp_source;

int ssrc_hash(uint32_t ssrc);

void        check_database(struct rtp *session);
rtp_source *get_source    (struct rtp *session, uint32_t ssrc);
rtp_source *create_source (struct rtp *session, uint32_t ssrc, int probation);
void        delete_source (struct rtp *session, uint32_t ssrc);

