/*
 * FILE:   rtp/rtp_tfrc.h
 * AUTHOR: Alvaro Saurin <saurin@dcs.gla.ac.uk>
 *
 * Copyright (c) 2005 University of Glasgow
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Revision: 1.4 $
 * $Date: 2005/12/16 16:22:36 $
 *
 */

#ifndef _RTP_TFRC_H
#define _RTP_TFRC_H

typedef struct {
        uint32_t	ts;
	uint32_t	tdelay;
	uint32_t	x_recv;
	uint32_t	p;
} rtcp_rx;				/* TFRC extensions */

typedef struct { 			/* used for pointer arithmetic */
        uint32_t        ssrc;           /* The ssrc to which this RR pertains */
#ifdef WORDS_BIGENDIAN
        uint32_t        fract_lost:8;
        uint32_t        total_lost:24;
#else
        uint32_t        total_lost:24;
        uint32_t        fract_lost:8;
#endif
        uint32_t        last_seq;
        uint32_t        jitter;
        uint32_t        lsr;
        uint32_t        dlsr;
	
        uint32_t        ts;
        uint32_t        tdelay;
        uint32_t        x_recv;
        uint32_t        p;
} rtcp_rr_rx;                              /* TFRC extensions */

/* Return TRUE if TFRC is been used */
int		rtp_tfrc_on		(struct rtp *session);

/* TFRC reception */
void		rtp_tfrc_recv_report	(struct rtp *session, rtcp_rx *rx);
void		rtp_tfrc_recv_data	(struct rtp *session, rtp_packet *packet);


/* TFRC formatting */
void		rtp_tfrc_format_rrxp	(struct rtp *session, rtcp_rr_rx *rrxp);
void		rtp_tfrc_format_data	(struct rtp *session, rtp_packet *packet);

/* Conversions */
unsigned int	rtp_tfrc_convert_tx	(rtp_packet *packet);
void		rtp_tfrc_convert_rx	(rtcp_rx *rx, rtcp_rr_rx *rrxp);

#endif

