/*
 * FILE:   rtp/sbuf.h
 * AUTHOR: Alvaro Saurin <saurin@dcs.gla.ac.uk>
 *
 * Copyright (c) 2005 University of Glasgow
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Revision: 1.10 $
 * $Date: 2006/02/17 14:27:51 $
 *
 */

#ifndef _SBUF_H
#define _SBUF_H

/* The sending buffer */
struct sbuf;
struct video_tx;

/*
 * The sending buffer context.
 * The context specifies some properties that the sending buffer must satisfy.
 * Any property with value equal to 0 will be ignored. Any conflict between
 * properties will be resolved in some unspecified way...
 */
struct sbuf_ctx_t {
	unsigned int	mtu;
	unsigned long	max_framelen;	/* max frame length (bytes) */
	unsigned int	max_out_ifi;	/* max time for transmitting a frame (usecs) */
	unsigned int	max_frames;	/* max number of frames in the buffer */
	unsigned int	max_frags;
};

/* 
 * External interface.
 */
struct sbuf		*sbuf_init	(struct sbuf_ctx_t	*ctx,
					 struct video_tx	*tx,
					 struct timeval		*base_time);
void			 sbuf_done	(struct sbuf		*send_buf);
bool			 sbuf_insert	(struct sbuf		*send_buf,
					 struct video_frame	*frame,
					 unsigned int		 frame_enc_id,
					 unsigned int		 encoder_id);
struct encoded_data	*sbuf_get_next	(struct sbuf		*send_buf);
unsigned int		 sbuf_length	(struct sbuf		*send_buf);
unsigned int		 sbuf_length_bytes	(struct sbuf		*send_buf);
bool			 sbuf_empty	(struct sbuf		*send_buf);
bool			 sbuf_remove	(struct sbuf		*send_buf,
					 struct timeval		 curr_time);

void			 sbuf_get_context (struct sbuf		*send_buf,
					   struct sbuf_ctx_t	*ctx);
void			 sbuf_set_context (struct sbuf		*send_buf,
					   struct sbuf_ctx_t	*ctx);


t_usec			 sbuf_get_in_ifi		(struct sbuf *send_buf);
t_usec			 sbuf_get_out_ifi		(struct sbuf *send_buf);

unsigned int		 sbuf_get_in_rate		(struct sbuf *send_buf);
unsigned int		 sbuf_get_comp_rate		(struct sbuf *send_buf);
unsigned int		 sbuf_get_out_rate		(struct sbuf *send_buf);

unsigned long		 sbuf_get_avg_in_frame_len	(struct sbuf *send_buf);
unsigned long		 sbuf_get_avg_out_frame_len	(struct sbuf *send_buf);

#endif
