/*
 * FILE:   trace.h
 * AUTHOR: Alvaro Saurin <saurin@dcs.gla.ac.uk>
 *
 * Copyright (c) 2005 University of Glasgow
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Revision: 1.8 $
 * $Date: 2006/02/06 13:33:54 $
 *
 */

#ifndef _TRACE_H
#define _TRACE_H

/* Standard events for trace collection. */
typedef unsigned int	trace_event_t; 
typedef double		trace_value_t;

/* 
 * Output type.
 * Traces can be dumped to a file or to a function
 */
typedef enum {
	t_none = 0,
	t_file,
	t_function,
} trace_output_t;

/* Type definition for a tracing function */
typedef void (*trace_fun_t)(struct timeval* ts, trace_event_t ev, trace_value_t val);

/* Inits the trace system, with a default trace file/function */
void	trace_init	(trace_output_t	  typ,
			 void		 *def_target,
			 char		**names,
			 char		**descs,
			 unsigned int	  len,
			 int		  rel_time);

/*
 * Registers a trace event definition, with a description and a
 * file or function. The target interpretation depends on the output type: if
 * it is a t_function, all trace events of this type will invoke the function
 * provided, or they will be logged to the target file if it is a t_file.
 */
void	trace_reg	(trace_event_t	 ev,
			 char		*name,
			 char		*desc,
			 trace_output_t	 typ,
			 void		*target);

void	trace_unreg	(trace_event_t	 ev);

/* Record an event in a given moment in time. */
void	trace		(trace_event_t);
void	trace_at	(struct timeval*, trace_event_t);
void	trace_val	(trace_event_t, trace_value_t);
void	trace_val_at	(struct timeval*, trace_event_t, trace_value_t);

/* Destroys the trace system */
void	trace_done	(void);

#endif

