/*
 * FILE:   transmit.h
 * AUTHOR: Colin Perkins <csp@isi.edu>
 *
 * Copyright (c) 2001-2002 University of Southern California
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 * 
 *      This product includes software developed by the University of Southern
 *      California Information Sciences Institute.
 * 
 * 4. Neither the name of the University nor of the Institute may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Revision: 1.12 $
 * $Date: 2006/04/25 12:42:43 $
 *
 */

#ifndef _TRANSMIT_H
#define _TRANSMIT_H

struct video_tx;

/* Delays */
#define MIN_FRAME_DELAY_SBUF	10000	/* Min transmit time for a frame (usecs)	*/
#define MAX_FRAME_DELAY		150000	/* Maximum delay for a frame (usecs)		*/

/* Transmission context */
struct tx_ctx {
	unsigned int	mtu;
	unsigned int	max_framelen;
	unsigned int	fps;
};

/* Transmitter initialization */
struct video_tx*	tx_init		(struct tx_ctx		*ctx,
					 struct rtp		*rtp_session,
					 struct vidcap		*capture_device,
					 unsigned int		 encoder_id,
					 unsigned int		 decoder_id,
					 struct timeval		*start_time,
					 struct cc_sndr_t	*cc_sndr,
					 unsigned int		*working);

/* Start the transmission system */
int			tx_start	(struct video_tx	*tx);

/* Stop the transmission system */
void			tx_stop		(struct video_tx	*tx);

/* Stop the transmitter and release resources */
void			tx_done		(struct video_tx	*tx);

/* Return TRUE if the transmitter is working */
unsigned int		tx_working	(struct video_tx	*tx);

/* Returns the expected display time of a frame */
t_usec			tx_trans_delay	(struct video_tx	*tx,
					 unsigned long		 datalen);

#endif
