/*
 * FILE:     tv.h
 * AUTHOR:   Colin Perkins <csp@csperkins.org>
 * MODIFIED: Ladan Gharai <ladan@isi.edu>
 *
 * Copyright (c) 2001-2003 University of Southern California
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 * 
 *      This product includes software developed by the University of Southern
 *      California Information Sciences Institute.
 * 
 * 4. Neither the name of the University nor of the Institute may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Revision: 1.23 $
 * $Date: 2006/02/13 16:33:26 $
 *
 */

#ifdef __cplusplus
extern "C" {
#endif

#ifndef __TV_H__
#define __TV_H__

#define TV_MIN_SLEEP		10000	/* Minimum sleep time (usecs) */
	
#ifdef HAVE_DARWIN
#define BUSY_WAIT_LIMIT		100	/* Maximum time to busy wait, in microseconds */
#else
#define BUSY_WAIT_LIMIT		1000
#endif

/*
 * Type types used.
 */
typedef double    t_sec;   /* Seconds type */
typedef uint32_t  t_usec;  /* Microseconds type */

/* Operations */
void     tv_cp		(struct timeval *dst, struct timeval *src);
t_sec    tv_diff	(struct timeval curr_time, struct timeval prev_time);
t_usec   tv_diff_usec	(struct timeval curr_time, struct timeval prev_time);
void     tv_add		(struct timeval *ts, t_sec offset_secs);
void	 tv_add_usec	(struct timeval *ts, t_usec usecs);
int      tv_gt		(struct timeval a, struct timeval b);
int      tv_eq		(struct timeval a, struct timeval b);

/* Sleep functions */
void	 tv_sleep_for	(t_usec t);
void	 tv_sleep_until	(struct timeval a);

/* Relative times */
uint32_t tv_mediatime	(void);
uint32_t tv_usecs_since	(struct timeval *base, struct timeval *t_now);

/* Conversions */
void	 tv2usec	(uint32_t *us, struct timeval tv);
void	 usec2tv	(struct timeval *tv, uint32_t us);
double	 usec2sec	(t_usec t);
t_usec	 sec2usec	(t_sec t);

#endif

#ifdef __cplusplus
}
#endif

