/*
 * FILE:   ui/ui_tcltk.c
 * AUTHOR: Alvaro Saurin <saurin@dcs.gla.ac.uk>
 *
 * Copyright (c) 2005 University of Glasgow
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Revision: 1.4 $
 * $Date: 2006/02/22 01:08:42 $
 *
 */

#include "config.h"
#include "config_unix.h"

/*
 * This code will be compiled if we have TCL/TK available and enabled...
 */ 
#ifdef HAVE_TCL_TK

#include "debug.h"

#include "ui/ui_tcltk.h"

#include <string.h>

#include "tcl.h"
#include "tk.h"

Tcl_Interp      *interp;

/*pthread_t      ui_thread;*/
/*pthread_mutex_t        ui_lock;*/
int              ui_is_done;
extern char      ui_script[];

#ifdef NDEF
static void *
ui_tcltk_thread_proc(void *arg)
{
        int     done;

        UNUSED(arg);

        do {
                while (Tcl_DoOneEvent(TCL_ALL_EVENTS | TCL_DONT_WAIT)) {
                        /* Process Tcl/Tk events */
                };
                //usleep(10000);
                pthread_mutex_lock(&ui_lock);
                done = ui_is_done;
                pthread_mutex_unlock(&ui_lock);
        } while (!done);
        return NULL;
}
#endif

/*
 * Probes the compatibility with the display given
 */ 
int
ui_tcltk_probe(const char* display) {
        /* Tcl/Tk accepts X11/XV as display device... */ 
        if( strcmp(display, "x11") == 0)        
                return 1;

        if( strcmp(display, "xv") == 0)
                return 1;

        if( strcmp(display, "none") == 0)
                return 1;
               
        return 0;
}


int
ui_tcltk_init(int argc, char *argv[])
{
        Tcl_Obj         *ui_script_obj;
        char            *cmd_line_args, buffer[10];

        Tcl_FindExecutable(argv[0]);
        interp = Tcl_CreateInterp();
        cmd_line_args = Tcl_Merge(argc - 1, argv + 1);
        Tcl_SetVar(interp, "argv", cmd_line_args, TCL_GLOBAL_ONLY);
        sprintf(buffer, "%d", argc - 1);
        Tcl_SetVar(interp, "argc", buffer, TCL_GLOBAL_ONLY);
        Tcl_SetVar(interp, "argv0", argv[0], TCL_GLOBAL_ONLY);
        Tcl_SetVar(interp, "tcl_interactive", "0", TCL_GLOBAL_ONLY);

        /* There is no easy way of preventing the Init functions from */
        /* loading the library files. Ignore error returns and load   */
        /* built in versions.                                         */
        if (Tcl_Init(interp) != TCL_OK) {
                printf("Unable to initialize user interface: %s\n", Tcl_GetStringResult(interp));
                return 1;
        }
        if (Tk_Init(interp) != TCL_OK) {
                printf("Unable to initialize user interface: %s\n", Tcl_GetStringResult(interp));
                return 1;
        }

        ui_script_obj = Tcl_NewStringObj(ui_script, strlen(ui_script));
        if (Tcl_EvalObj(interp, ui_script_obj) != TCL_OK) {
                printf("Unable to initialize user interface: %s\n", Tcl_GetStringResult(interp));
                return 1;
        }

        while (Tcl_DoOneEvent(TCL_ALL_EVENTS | TCL_DONT_WAIT)) {
                /* Process Tcl/Tk events */
        }       
        Tcl_ResetResult(interp);

        ui_is_done = 0;
#ifdef NDEF
        pthread_mutex_init(&ui_lock, NULL);
        if (pthread_create(&ui_thread, NULL, ui_tcltk_thread_proc, NULL) != 0) {
                perror("Unable to initialize user interface\n");
                return 1;
        }
#endif

        return 0;
}

void
ui_tcltk_done(void)
{
/*      pthread_mutex_lock(&ui_lock);*/
        ui_is_done = 1;
/*      pthread_mutex_unlock(&ui_lock);*/
/*      pthread_join(ui_thread, NULL);*/
}

#endif /* HAVE_TCL_TK */

