/*
 * FILE:   testcard.c
 * AUTHOR: Colin Perkins <csp@csperkins.org>
 *
 * Copyright (c) 2004-2005 University of Glasgow
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 * 
 *      This product includes software developed by the University of Southern
 *      California Information Sciences Institute.
 * 
 * 4. Neither the name of the University nor of the Institute may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Revision: 1.19 $
 * $Date: 2006/02/15 15:05:39 $
 *
 */

#include "config.h"
#include "config_unix.h"
#include "debug.h"
#include "tv.h"
#include "video_types.h"
#include "video_capture.h"
#include "video_capture/testcard.h"

extern unsigned char	testcard_image[];

struct testcard_state {
	struct timeval	next_send_time;
	int		fps;
	int		count;
	unsigned int	period;		/* usecs between frames */
};

void *
vidcap_testcard_init(capture_format_t *format, const char *args)
{
	struct testcard_state	*s;
        
	UNUSED(args);
	
        assert( format != NULL);
        
	s = malloc(sizeof(struct testcard_state));
	if ( s != NULL ) {
		s->fps		= format[0].fps;
		s->count	= 0;
		s->period	= 1.0/(double) s->fps;
		
		gettimeofday (&(s->next_send_time), NULL);
		tv_add (&(s->next_send_time), s->period);
	}
	return s;
}

void
vidcap_testcard_done(void *state)
{
	free (state);
}

struct video_frame *
vidcap_testcard_grab(void *arg)
{
	struct testcard_state	*state;
	struct video_frame	*vf;

	state = (struct testcard_state *) arg;

	tv_sleep_until (state->next_send_time);
	gettimeofday (&(state->next_send_time), NULL);
	tv_add (&(state->next_send_time), state->period);

	state->count++;

	vf = (struct video_frame *) malloc(sizeof(struct video_frame));
	if (vf != NULL) {
		vf->colour_mode = YUV_422;
		vf->width       = 1280;
		vf->height      =  720;
		vf->data        = testcard_image;
		vf->data_len	= 1280 * 720 * 2;
	}
	return vf;
}

struct vidcap_type *
vidcap_testcard_probe (const struct run_env* environ)
{
	struct vidcap_type	*vt;
	capture_format_t	*cformat;
        
        assert(environ != NULL);        
	
	cformat = malloc(sizeof(capture_format_t));
	if(cformat != NULL) {
		cformat[0].size        = DS_720p;
		cformat[0].colour_mode = YUV_422;
		cformat[0].fps         = 25;
		cformat[0].encoding    = fe_raw;		
	} else {
		return NULL;
	}		
	
	vt = (struct vidcap_type *) malloc(sizeof(struct vidcap_type));
	if (vt != NULL) {
		vt->id          = VIDCAP_TESTCARD_ID;
		vt->name        = "testcard";
		vt->description = "Video testcard";
		vt->props       = 0;
		vt->formats	= cformat;
		vt->num_formats = 1;
	}
	return vt;
}

