/*
 * FILE:   video_codec/dv.h
 * AUTHOR: Colin Perkins <csp@csperkins.org>
 *
 * Copyright (c) 2004 University of Glasgow
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Revision: 1.21 $
 * $Date: 2005/10/12 15:52:28 $
 *
 */

#ifndef _DV_H
#define _DV_H

#define VCODEC_DV_ID	0x03a7adb8

int			 dv_codec_init	(void);
void			 dv_codec_exit	(void);

const char		*dv_codec_get_name	(void);
const char		*dv_codec_get_desc	(void);

int			 dv_codec_can_encode    (void);
int			 dv_codec_can_packetize (void);
int			 dv_codec_can_decode    (void);

/* Encoded frames management */
struct encoded_frame	*dv_eframe_alloc    (void);
struct encoded_data	*dv_eframe_get      (struct encoded_frame	*eframe,
					     unsigned int		 pos);
void			 dv_eframe_free     (struct encoded_frame	*eframe);

/* Encode and packetize */
void                *dv_encoder_create  (unsigned id, unsigned mtu);
void                 dv_encoder_destroy (void *state);
unsigned int         dv_encoder_get_framelen(void);
int                  dv_encode		(void                 *state, 
					 struct video_frame   *in, 
					 struct encoded_frame *out,
					 struct vcodec_ctx_t  *ctx);
int                  dv_packetize	(void                 *state, 
					 struct video_frame   *in, 
					 struct encoded_frame *out);

/* Decoder info */
video_colour_mode_t  dv_decoder_get_cm		(void *state);
display_size_t       dv_decoder_get_size	(void *state);
unsigned int         dv_decoder_get_framelen	(void *state);

/* Decoding */
void                *dv_decoder_create	(unsigned id);
void                 dv_decoder_destroy	(void *state);
int                  dv_decode		(void *state, 
					 struct coded_data *in, 
					 struct video_frame *out);
int                  dv_decode_raw	(void *state, 
					 struct video_frame *in,
					 struct video_frame *out);
int                  dv_decode_partial	(void *state, 
					 struct coded_data *in,
					 struct video_frame *out);

#endif /* _DV_H */

