/*
 * FILE:   video_codec/jpeg_aux.h
 * AUTHOR: Alvaro Saurin <saurin@dcs.gla.ac.uk>
 *
 * Copyright (c) 2005 University of Glasgow
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Revision: 1.2 $
 * $Date: 2005/11/08 14:32:56 $
 *
 */

#ifndef _JPEG_AUX_H
#define _JPEG_AUX_H

struct libjpeg_enc_state;
struct libjpeg_dec_state;

/*
 * Max number of fragments for a frame:
 * max_frags <= ((max_frame_size * max_bytes_per_pixel) / max_MTU)
 */
#define LIBJPEG_MAX_FRAGS	(((1920 * 1080 * 3) / 1500) + 1)

/*
 * Output of the compression. The encoder returns this struct, filled with all
 * the fragments (all of the same size, MTU). The user of the compress function
 * is responsible for freeing all the memory for all fragments and structure.
 */
struct jpeg_enc_outdata {
	unsigned long	 datalen;
	unsigned int	 num_frags;
	unsigned char	*frags [LIBJPEG_MAX_FRAGS];
	unsigned int	 frags_len [LIBJPEG_MAX_FRAGS];
};

/* Initialize the compression system */
struct libjpeg_enc_state
		*libjpeg_compress_init		(unsigned int			 width,
						 unsigned int			 height,			
						 video_colour_mode_t		 color,
						 unsigned int			 mtu);
/* Destroy the compression system */
void		 libjpeg_compress_done		(struct libjpeg_enc_state	*s);

/* Compress a frame */
struct jpeg_enc_outdata*
		libjpeg_compress		(struct libjpeg_enc_state	*s,
						 unsigned int			 target_size,
						 unsigned char			*in_data,
						 unsigned int			 in_data_len);

/* Initialize the decompression system */
struct libjpeg_dec_state
		*libjpeg_decompress_init	(void);

/* Destroy the decompression system */
void		 libjpeg_decompress_done	(struct libjpeg_dec_state	*s);

/* Decompress a frame */
int		 libjpeg_decompress		(struct libjpeg_dec_state	*s,
						 struct coded_data		*in_ptr,
						 struct video_frame		*out);

#endif

