/*
 * FILE:   video_codec/mpeg1.h
 * AUTHOR: Alvaro Saurin <saurin@dcs.gla.ac.uk>
 *
 * Copyright (c) 2005 University of Glasgow
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Revision: 1.1 $
 * $Date: 2006/05/12 15:00:57 $
 *
 */

#ifndef _MPEG1_H
#define _MPEG1_H

#define VCODEC_MPEG1_ID	0x03b3cdb8

int			 mpeg1_codec_init		(void);
void			 mpeg1_codec_exit		(void);

const char		*mpeg1_codec_get_name		(void);
const char		*mpeg1_codec_get_desc		(void);

int			 mpeg1_codec_can_encode		(void);
int			 mpeg1_codec_can_packetize	(void);
int			 mpeg1_codec_can_decode		(void);

/* Encoded frames management */
struct encoded_frame	*mpeg1_eframe_alloc	(void);
struct encoded_data	*mpeg1_eframe_get	(struct encoded_frame	*eframe,
						 unsigned int		 pos);
void			 mpeg1_eframe_free	(struct encoded_frame	*eframe);

/* Encode and packetize */
void                *mpeg1_encoder_create	(unsigned id, unsigned mtu);
void                 mpeg1_encoder_destroy	(void *state);
unsigned int         mpeg1_encoder_get_framelen	(void);
int                  mpeg1_encode		(void                 *state, 
						 struct video_frame   *in, 
						 struct encoded_frame *out,
						 struct vcodec_ctx_t  *ctx);
int                  mpeg1_packetize		(void                 *state, 
						 struct video_frame   *in, 
						 struct encoded_frame *out);

/* Decoder info */
video_colour_mode_t  mpeg1_decoder_get_cm	(void *state);
display_size_t       mpeg1_decoder_get_size	(void *state);
unsigned int         mpeg1_decoder_get_framelen	(void *state);

/* Decoding */
void                *mpeg1_decoder_create	(unsigned id);
void                 mpeg1_decoder_destroy	(void *state);
int                  mpeg1_decode		(void *state, 
						 struct coded_data *in, 
						 struct video_frame *out);
int                  mpeg1_decode_raw		(void *state, 
						 struct video_frame *in,
						 struct video_frame *out);
int                  mpeg1_decode_partial	(void *state, 
						 struct coded_data *in,
						 struct video_frame *out);

#endif /* _DV_H */

