/*
 * FILE:   video_codec/uv_yuv422.h
 * AUTHOR: Colin Perkins <csp@csperkins.org>
 *
 * Copyright (c) 2004-2005 University of Glasgow
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Revision: 1.1 $
 * $Date: 2005/12/19 13:47:13 $
 *
 */

#ifndef _UV_YUV_422_H
#define _UV_YUV_422_H

#define VCODEC_YUV422_ID	0x04a6bdb8

/* Codec management */
int                  uv_yuv422_init		(void);
void                 uv_yuv422_exit		(void);

/* Codec capabilities */
const char *         uv_yuv422_get_name	(void);
const char *         uv_yuv422_get_desc	(void);
int                  uv_yuv422_can_encode	(void);
int                  uv_yuv422_can_packetize	(void);
int                  uv_yuv422_can_decode	(void);

/* Encoded frames management */
struct encoded_frame	*uv_yuv422_eframe_alloc (void);
struct encoded_data	*uv_yuv422_eframe_get	(struct encoded_frame	*eframe,
						 unsigned int		 pos);
void			uv_yuv422_eframe_free (struct encoded_frame	*eframe);

/* Encoding and packetization */
void                *uv_yuv422_encoder_create	(unsigned id, unsigned mtu);
void                 uv_yuv422_encoder_destroy(void *state);
unsigned int         uv_yuv422_encoder_get_framelen	(void);
int                  uv_yuv422_encode		(void *state, 
						 struct video_frame	*in,
						 struct encoded_frame	*out,
						 struct vcodec_ctx_t	*ctx);
int                  uv_yuv422_packetize	(void			*state, 
						 struct video_frame	*in,
						 struct encoded_frame	*out);

/* Decoding */
video_colour_mode_t  uv_yuv422_decoder_get_cm		(void *state);
display_size_t       uv_yuv422_decoder_get_size	(void *state);
unsigned int         uv_yuv422_decoder_get_framelen	(void *state);

void                *uv_yuv422_decoder_create		(unsigned id);
void                 uv_yuv422_decoder_destroy	(void *state);
int                  uv_yuv422_decode			(void *state, 
							 struct coded_data *in,
							 struct video_frame *out);
int                  uv_yuv422_decode_raw		(void *state, 
							 struct video_frame *in,
							 struct video_frame *out);
int                  uv_yuv422_decode_partial		(void *state, 
							 struct coded_data *in,
							 struct video_frame *out);

#endif /* _UV_VUY_422_8_H */
