/*
 * FILE:   yv12_to_yuv422.c
 * AUTHOR: Alvaro Saurin <saurin@dcs.gla.ac.uk>
 *
 * Copyright (c) 2005 University of Glasgow
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Revision: 1.1 $
 * $Date: 2005/11/14 13:05:43 $
 *
 */

#include "config.h"
#include "config_unix.h"
#include "debug.h"
#include "video_types.h"
#include "video_convert/yv12_to_yuv422.h"

void
yv12_to_yuv422 (struct video_frame *src, struct video_frame *dst)
{
	unsigned int	width, height;
	unsigned char	*dst_ptr;
	unsigned char	*src_ptr[2];
	unsigned int	 i, j;
	
	assert (src		!= NULL);
	assert (src->data	!= NULL);
	assert (dst		!= NULL);
	assert (dst->data	!= NULL);
	
	width		= src->width;
	height		= src->height;

	assert (dst->data_len	>= width * height * cm_get_bpp(YUV_422));

	dst_ptr		= dst->data;
	src_ptr[0]	= src->data;
	src_ptr[1]	= src_ptr[0] + width*height;
	src_ptr[2]	= src_ptr[1] + (width*height)/2;
	
	for (i = 0; i < height; i += 2) {
		for (j = 0; j < width/2; j++) {
			/* skip every second line for U and V */
			dst_ptr++;
			*(dst_ptr++) = *(src_ptr[0]++);
			dst_ptr++;
			*(dst_ptr++) = *(src_ptr[0]++);
		}

		for (j = 0; j < width/2; j++) {
			/* packed YUV 422 = U[i] Y[i] V[i] Y[i+1]  */
			*(dst_ptr++) = *(src_ptr[1]++);
			*(dst_ptr++) = *(src_ptr[0]++);
			*(dst_ptr++) = *(src_ptr[2]++);
			*(dst_ptr++) = *(src_ptr[0]++);
		}
	}	
}


