/*
 * FILE:    run_tests.c
 * AUTHORS: Colin Perkins
 *
 * Copyright (c) 2004 University of Glasgow
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted provided that the following conditions 
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      Glasgow Department of Computing Science
 * 4. Neither the name of the University nor of the Department may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Revision: 1.19 $
 * $Date: 2005/12/19 13:45:07 $
 */

#include "config.h"
#include "config_unix.h"
#include "debug.h"
#include "test_audio_hw.h"
#include "test_tv.h"
#include "test_bitstream.h"
#include "test_audio_codec_dvi.h"
#include "test_audio_codec_g711.h"
#include "test_audio_codec_g726.h"
#include "test_audio_codec_gsm.h"
#include "test_audio_codec_l16.h"
#include "test_audio_codec_l8.h"
#include "test_audio_codec_lpc.h"
#include "test_audio_codec_vdvi.h"
#include "test_video_codec_jpeg.h"
#include "test_video_codec_dv.h"
#include "test_video_codec_uv_yuv422.h"
#include "test_random.h"
#include "test_sbuf.h"
#include "test_pbuf.h"
#include "test_net_udp.h"
#include "test_rtp.h"
#include "test_des.h"
#include "test_aes.h"
#include "test_md5.h"
#include "test_video_codecs.h"
#include "test_video_capture.h"
#include "test_video_display.h"

uint32_t	RTT;	/* FIXME: will be removed once the global in main.c is removed */

#ifdef main
#undef main
#endif

int
main(int argc, char *argv[])
{
	int	rc;

	UNUSED(argc);
	UNUSED(argv);

#ifndef DEBUG
	printf("WARNING: Certain tests are ineffective unless debugging is enabled!\n");
#endif

	rc  = test_bitstream();
	rc |= test_audio_codec_dvi();
	rc |= test_audio_codec_g711();
	rc |= test_audio_codec_g726();
	rc |= test_audio_codec_gsm();
	rc |= test_audio_codec_l16();
	rc |= test_audio_codec_l8();
	rc |= test_audio_codec_lpc();
	rc |= test_audio_codec_vdvi();
	rc |= test_video_codecs();
	rc |= test_video_codec_jpeg();
	rc |= test_video_codec_dv();
	rc |= test_video_codec_uv_yuv422();
	rc |= test_random();
	rc |= test_sbuf();
	rc |= test_pbuf();
	rc |= test_net_udp();
	rc |= test_tv();
	rc |= test_rtp();
	rc |= test_des();
	rc |= test_aes();
	rc |= test_md5();

#ifdef TEST_AV_HW
	rc |= test_video_capture();
	rc |= test_video_display();
	rc |= test_audio_hw();
#endif

	if (rc != 0) {
		printf("\n");
		printf("*** Some tests failed!\n");
		printf("\n");
	}
	return rc;
}


