/*
 * FILE:    test_audio_codec_gsm.c
 * AUTHORS: Colin Perkins
 *
 * Test the GSM codec
 *
 * Copyright (c) 2004-2005 University of Glasgow
 * Copyright (c) 2003-2004 University of Southern California
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted provided that the following conditions 
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "config.h"
#include "config_unix.h"
#include "debug.h"
#include "audio_types.h"
#include "audio_codec_types.h"
#include "audio_codec/gsm.h"
#include "audio_codec/gsm_impl.h"
#include "test_audio_codec_gsm.h"

#ifdef REALLY_TEST_GSM 

static int 
gsm_explode(gsm_byte *c, gsm_signal *target)
{
#	define	LARc	target
#	define	Nc	*((gsm_signal (*) [17])(target + 8))
#	define	bc	*((gsm_signal (*) [17])(target + 9))
#	define	Mc	*((gsm_signal (*) [17])(target + 10))
#	define	xmaxc	*((gsm_signal (*) [17])(target + 11))

	/* GSM_MAGIC  = (*c >> 4) & 0xF; */

	if (((*c >> 4) & 0x0F) != GSM_MAGIC) return -1;

	LARc[0]  = (*c++ & 0xF) << 2;		/* 1 */
	LARc[0] |= (*c >> 6) & 0x3;
	LARc[1]  = *c++ & 0x3F;
	LARc[2]  = (*c >> 3) & 0x1F;
	LARc[3]  = (*c++ & 0x7) << 2;
	LARc[3] |= (*c >> 6) & 0x3;
	LARc[4]  = (*c >> 2) & 0xF;
	LARc[5]  = (*c++ & 0x3) << 2;
	LARc[5] |= (*c >> 6) & 0x3;
	LARc[6]  = (*c >> 3) & 0x7;
	LARc[7]  = *c++ & 0x7;

	Nc[0]  = (*c >> 1) & 0x7F;

	bc[0]  = (*c++ & 0x1) << 1;
	bc[0] |= (*c >> 7) & 0x1;

	Mc[0]  = (*c >> 5) & 0x3;

	xmaxc[0]  = (*c++ & 0x1F) << 1;
	xmaxc[0] |= (*c >> 7) & 0x1;

#undef	xmc
#define	xmc	(target + 12)

	xmc[0]  = (*c >> 4) & 0x7;
	xmc[1]  = (*c >> 1) & 0x7;
	xmc[2]  = (*c++ & 0x1) << 2;
	xmc[2] |= (*c >> 6) & 0x3;
	xmc[3]  = (*c >> 3) & 0x7;
	xmc[4]  = *c++ & 0x7;
	xmc[5]  = (*c >> 5) & 0x7;
	xmc[6]  = (*c >> 2) & 0x7;
	xmc[7]  = (*c++ & 0x3) << 1;		/* 10 */
	xmc[7] |= (*c >> 7) & 0x1;
	xmc[8]  = (*c >> 4) & 0x7;
	xmc[9]  = (*c >> 1) & 0x7;
	xmc[10]  = (*c++ & 0x1) << 2;
	xmc[10] |= (*c >> 6) & 0x3;
	xmc[11]  = (*c >> 3) & 0x7;
	xmc[12]  = *c++ & 0x7;

	Nc[1]  = (*c >> 1) & 0x7F;

	bc[1]  = (*c++ & 0x1) << 1;
	bc[1] |= (*c >> 7) & 0x1;

	Mc[1]  = (*c >> 5) & 0x3;

	xmaxc[1]  = (*c++ & 0x1F) << 1;
	xmaxc[1] |= (*c >> 7) & 0x1;

#undef	xmc
#define	xmc	(target + 29 - 13)

	xmc[13]  = (*c >> 4) & 0x7;
	xmc[14]  = (*c >> 1) & 0x7;
	xmc[15]  = (*c++ & 0x1) << 2;
	xmc[15] |= (*c >> 6) & 0x3;
	xmc[16]  = (*c >> 3) & 0x7;
	xmc[17]  = *c++ & 0x7;
	xmc[18]  = (*c >> 5) & 0x7;
	xmc[19]  = (*c >> 2) & 0x7;
	xmc[20]  = (*c++ & 0x3) << 1;
	xmc[20] |= (*c >> 7) & 0x1;
	xmc[21]  = (*c >> 4) & 0x7;
	xmc[22]  = (*c >> 1) & 0x7;
	xmc[23]  = (*c++ & 0x1) << 2;
	xmc[23] |= (*c >> 6) & 0x3;
	xmc[24]  = (*c >> 3) & 0x7;
	xmc[25]  = *c++ & 0x7;

	Nc[2]  = (*c >> 1) & 0x7F;

	bc[2]  = (*c++ & 0x1) << 1;		/* 20 */
	bc[2] |= (*c >> 7) & 0x1;

	Mc[2]  = (*c >> 5) & 0x3;

	xmaxc[2]  = (*c++ & 0x1F) << 1;
	xmaxc[2] |= (*c >> 7) & 0x1;

#undef	xmc
#define	xmc	(target + 46 - 26)

	xmc[26]  = (*c >> 4) & 0x7;
	xmc[27]  = (*c >> 1) & 0x7;
	xmc[28]  = (*c++ & 0x1) << 2;
	xmc[28] |= (*c >> 6) & 0x3;
	xmc[29]  = (*c >> 3) & 0x7;
	xmc[30]  = *c++ & 0x7;
	xmc[31]  = (*c >> 5) & 0x7;
	xmc[32]  = (*c >> 2) & 0x7;
	xmc[33]  = (*c++ & 0x3) << 1;
	xmc[33] |= (*c >> 7) & 0x1;
	xmc[34]  = (*c >> 4) & 0x7;
	xmc[35]  = (*c >> 1) & 0x7;
	xmc[36]  = (*c++ & 0x1) << 2;
	xmc[36] |= (*c >> 6) & 0x3;
	xmc[37]  = (*c >> 3) & 0x7;
	xmc[38]  = *c++ & 0x7;

	Nc[3]  = (*c >> 1) & 0x7F;

	bc[3]  = (*c++ & 0x1) << 1;
	bc[3] |= (*c >> 7) & 0x1;

	Mc[3]  = (*c >> 5) & 0x3;

	xmaxc[3]  = (*c++ & 0x1F) << 1;
	xmaxc[3] |= (*c >> 7) & 0x1;

#undef	xmc
#define	xmc	(target + 63 - 39)

	xmc[39]  = (*c >> 4) & 0x7;
	xmc[40]  = (*c >> 1) & 0x7;
	xmc[41]  = (*c++ & 0x1) << 2;
	xmc[41] |= (*c >> 6) & 0x3;
	xmc[42]  = (*c >> 3) & 0x7;
	xmc[43]  = *c++ & 0x7;			/* 30  */
	xmc[44]  = (*c >> 5) & 0x7;
	xmc[45]  = (*c >> 2) & 0x7;
	xmc[46]  = (*c++ & 0x3) << 1;
	xmc[46] |= (*c >> 7) & 0x1;
	xmc[47]  = (*c >> 4) & 0x7;
	xmc[48]  = (*c >> 1) & 0x7;
	xmc[49]  = (*c++ & 0x1) << 2;
	xmc[49] |= (*c >> 6) & 0x3;
	xmc[50]  = (*c >> 3) & 0x7;
	xmc[51]  = *c & 0x7;			/* 33 */

	return 0;
}

static void
dump_coded_data(unsigned short *d1, unsigned short *d2)
{
	int	i;

	printf("  Got : ");
	for (i = 0; i < 76; i++) {
		printf("%04hx ", d1[i]);
	}
	printf("\n");

	printf("  Want: ");
	for (i = 0; i < 76; i++) {
		printf("%04hx ", d2[i]);
	}
	printf("\n");
}

#endif

int 
test_audio_codec_gsm(void)
{
	int		 	 i, nfmts, fmt;
	const acodec_format_t	*cf;
#ifdef REALLY_TEST_GSM
	unsigned char			*state_e, *state_d;
	int			 inp_file, out_file, cod_file;
	sample			 inp[160], out[160], du[160];
	short		 	 cod[76], cu_e[76];
	coded_unit		 cu;
	int			 rc;
#endif

	printf("Testing audio codec: GSM "); 

	/* Find the standard 8kHz version of the GSM codec... */
	nfmts = gsm_get_formats_count();
	for (i = 0; i < nfmts; i++) {
		cf = gsm_get_format(i);
		if (strcmp(cf->long_name, "GSM-8K-Mono") == 0) {
			fmt = i;
			goto found_codec;
		}
	}
	printf("FAIL\n");
	printf("  Cannot initialize codec\n");
	return 1;

found_codec:

#ifdef REALLY_TEST_GSM
	/*********************************************************************/
	/* Run test pattern 1: 584 frames                                    */
	/*********************************************************************/
	printf("......... ");
	gsm_state_create(fmt, &state_e);
	gsm_state_create(fmt, &state_d);

	inp_file = open("test/test_audio_codec_gsm.seq01.inp", O_RDONLY, 0);
	out_file = open("test/test_audio_codec_gsm.seq01.out", O_RDONLY, 0);
	cod_file = open("test/test_audio_codec_gsm.seq01.cod", O_RDONLY, 0);

	for (i = 0; i < 584; i++) {
		rc  = read(inp_file, inp, 320);
		rc += read(cod_file, cod, 152);
		rc += read(out_file, out, 320);
		if (rc != 792) {
			printf(" FAIL\n");
			perror("  Cannot read test vectors:");
			return 1;
		}

		gsm_encoder(fmt, state_e, inp, &cu);
		gsm_explode(cu.data, cu_e);
		if (memcmp(cu_e, cod, 152) != 0) {
			printf(" FAIL\n");
			printf("  Encoding failed on frame %d\n", i);
			dump_coded_data(cu_e, cod);
			return 1;
		}	

		gsm_decoder(fmt, state_d, &cu, du);
		if (memcmp(du, out, 320) != 0) {
			printf(" FAIL\n");
			printf("  Decoding failed on frame %d\n", i);
			return 1;
		}
	}

	close(inp_file);
	close(out_file);
	close(cod_file);

	gsm_state_destroy(fmt, &state_e);
	gsm_state_destroy(fmt, &state_d);

	/*********************************************************************/
	/* Run test pattern 2                                                */
	/*********************************************************************/

	/*********************************************************************/
	/* Run test pattern 3                                                */
	/*********************************************************************/

	/*********************************************************************/
	/* Run test pattern 4                                                */
	/*********************************************************************/

	/*********************************************************************/
	/* Run test pattern 5                                                */
	/*********************************************************************/

	/*********************************************************************/
#else
	printf("................................................. ");
#endif
	printf("--\n");
        return 0;
}

