/*
 * FILE:    test_pbuf.c
 * AUTHORS: Colin Perkins
 *
 * Copyright (c) 2004 University of Glasgow
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted provided that the following conditions 
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "config.h"
#include "config_unix.h"
#include "debug.h"
#include "video_types.h"
#include "video_display.h"
#include "rtp/cc.h"
#include "rtp/rtp.h"
#include "rtp/pbuf.h"
#include "test_pbuf.h"

int 
test_pbuf(void)
{
	struct timeval	 curr_time;
	struct pbuf	*pb;
	rtp_packet	*rp[5];

	printf("Testing playout buffer (sequence number comparisons) ..................... "); 
	fflush(stdout);

	if (!seq_lt(1000u, 1001u)) {
		printf("FAIL\n");
		printf("  Reported 1000 > 1001 modulo 65535\n");
		return 1;
	}
	if (seq_lt(300u, 250u)) {
		printf("FAIL\n");
		printf("  Reported 300 < 250 modulo 65535\n");
		return 1;
	}
	if (!seq_lt(65535u, 0u)) {
		printf("FAIL\n");
		printf("  Reported 65535 > 0 modulo 65535\n");
		return 1;
	}
	if (seq_lt(0u, 65535u)) {
		printf("FAIL\n");
		printf("  Reported 0 < 65535 modulo 65535\n");
		return 1;
	}
	if (!seq_lt(48116u, 48117u)) {
		printf("FAIL\n");
		printf("  Reported 48116 > 48117 modulo 65535\n");
		return 1;
	}
	printf("Ok\n");

	printf("Testing playout buffer (wrapped sequence number/reordering) .............. "); 
	fflush(stdout);

	/* Test 1: Insert packets into the playout buffer, with the sequence  */
	/* number wrapping around and with out of order packets. The playout  */
	/* buffer stores packets in order of RTP sequence number; this test   */
	/* is intended to expose bugs in the code to handle packet insertion. */ 

	pb = pbuf_init();

	rp[0] = malloc(sizeof(rtp_packet));
	rp[0]->meta.csrc      = NULL;
	rp[0]->meta.data      = NULL;
	rp[0]->meta.data_len  = 0;
	rp[0]->meta.extn      = NULL;
	rp[0]->meta.extn_len  = 0;
	rp[0]->fields.v       = 2;
	rp[0]->fields.p       = 0;
	rp[0]->fields.x       = 0;
	rp[0]->fields.cc      = 0;
	rp[0]->fields.m       = 0;
	rp[0]->fields.pt      = 0;
	rp[0]->fields.seq     = 0xffff;
	rp[0]->fields.ts      = 1000;
	rp[0]->fields.ssrc    = 0x12345678;
	rp[0]->fields.send_ts = 0;
	rp[0]->fields.rtt     = 0;
	pbuf_insert(pb, rp[0]);

	rp[1] = malloc(sizeof(rtp_packet));
	rp[1]->meta.csrc      = NULL;
	rp[1]->meta.data      = NULL;
	rp[1]->meta.data_len  = 0;
	rp[1]->meta.extn      = NULL;
	rp[1]->meta.extn_len  = 0;
	rp[1]->fields.v       = 2;
	rp[1]->fields.p       = 0;
	rp[1]->fields.x       = 0;
	rp[1]->fields.cc      = 0;
	rp[1]->fields.m       = 0;
	rp[1]->fields.pt      = 0;
	rp[1]->fields.seq     = 0x0000;
	rp[1]->fields.ts      = 1000;
	rp[1]->fields.ssrc    = 0x12345678;
	rp[1]->fields.send_ts = 0;
	rp[1]->fields.rtt     = 0;
	pbuf_insert(pb, rp[1]);

	rp[2] = malloc(sizeof(rtp_packet));
	rp[2]->meta.csrc      = NULL;
	rp[2]->meta.data      = NULL;
	rp[2]->meta.data_len  = 0;
	rp[2]->meta.extn      = NULL;
	rp[2]->meta.extn_len  = 0;
	rp[2]->fields.v       = 2;
	rp[2]->fields.p       = 0;
	rp[2]->fields.x       = 0;
	rp[2]->fields.cc      = 0;
	rp[2]->fields.m       = 0;
	rp[2]->fields.pt      = 0;
	rp[2]->fields.seq     = 0x0002;
	rp[2]->fields.ts      = 1000;
	rp[2]->fields.ssrc    = 0x12345678;
	rp[2]->fields.send_ts = 0;
	rp[2]->fields.rtt     = 0;
	pbuf_insert(pb, rp[2]);

	rp[3] = malloc(sizeof(rtp_packet));
	rp[3]->meta.csrc      = NULL;
	rp[3]->meta.data      = NULL;
	rp[3]->meta.data_len  = 0;
	rp[3]->meta.extn      = NULL;
	rp[3]->meta.extn_len  = 0;
	rp[3]->fields.v       = 2;
	rp[3]->fields.p       = 0;
	rp[3]->fields.x       = 0;
	rp[3]->fields.cc      = 0;
	rp[3]->fields.m       = 0;
	rp[3]->fields.pt      = 0;
	rp[3]->fields.seq     = 0x0001;
	rp[3]->fields.ts      = 1000;
	rp[3]->fields.ssrc    = 0x12345678;
	rp[3]->fields.send_ts = 0;
	rp[3]->fields.rtt     = 0;
	pbuf_insert(pb, rp[3]);

	rp[4] = malloc(sizeof(rtp_packet));
	rp[4]->meta.csrc      = NULL;
	rp[4]->meta.data      = NULL;
	rp[4]->meta.data_len  = 0;
	rp[4]->meta.extn      = NULL;
	rp[4]->meta.extn_len  = 0;
	rp[4]->fields.v       = 2;
	rp[4]->fields.p       = 0;
	rp[4]->fields.x       = 0;
	rp[4]->fields.cc      = 0;
	rp[4]->fields.m       = 0;
	rp[4]->fields.pt      = 0;
	rp[4]->fields.seq     = 0x0003;
	rp[4]->fields.ts      = 1000;
	rp[4]->fields.ssrc    = 0x12345678;
	rp[4]->fields.send_ts = 0;
	rp[4]->fields.rtt     = 0;
	pbuf_insert(pb, rp[4]);

	usleep(500000);	/* Allow playout time to pass... */
	gettimeofday(&curr_time, NULL);
	pbuf_remove(pb, curr_time);
	pbuf_done(pb);

	printf("Ok\n");
	return 0;
}

