/*
 * FILE:    test_video_capture.c
 * AUTHORS: Colin Perkins
 *
 * Test video hardware probing routines
 *
 * Copyright (c) 2003-2004 University of Southern California
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted provided that the following conditions 
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Computer Science
 *      Department at University College London
 * 4. Neither the name of the University nor of the Department may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "config.h"
#include "config_unix.h"
#include "debug.h"
#include "environment.h"
#include "memory.h"
#include "video_types.h"
#include "video_capture.h"
#include "test_video_capture.h"

int 
test_video_capture(void)
{
	struct vidcap_type	*vt;
	unsigned		 i;
        struct run_env*           environ;
        
        environ = env_init();
	
	printf("Testing video capture device detection ................................... "); 
	if (vidcap_init_devices( environ ) != 0) {
		printf("FAIL\n");
		printf("    Cannot probe video capture devices\n");
		return 1;
	}
	printf("Ok\n");

	for (i = 0; i < vidcap_get_device_count(); i++) {
		vt = vidcap_get_device_details(i);
		printf("    \"%s\"\n", vt->name);
		printf("        description: %s\n", vt->description);
		printf("        image size : %dx%d\n", 
		       display_get_width(vt->formats[0].size), 
		       display_get_height(vt->formats[0].size));
	}
	vidcap_free_devices();
	return 0;
}

