#! /bin/sh
#
# Calculate the average for an event 'event', every 'granularity' ms.
#
# Output format: <TIME> <NUM-SAMPLES> <AVERAGE>

PATTERN="$1"
FILE=$2
GRAN=$3

DEF_GRAN=60		# Default granularity (in ms)

################################################################################
# Find the average for a event (with the granularity provided)
#
avg() {
	EV_STR="s/$1//"
	DESC_STR="s/[\'\"].*[\'\"]//"

	grep $1 $2 | sed -e $EV_STR | sed -e $DESC_STR |
	awk -v gran_init=$3 '
	BEGIN   {
		acum_size	= 0
		granularity	= gran_init
		num_samples	= 0
	}	
	NR == 1	{
		rel_time	= 0
		event_time 	= ($1 * 1000)		
		min_time	= event_time
		max_time	= min_time + granularity
	}
		{
		event_time 	= ($1 * 1000)
		amount		= ($2 * 1000)
		
		# printf ("%.f %.f\n", event_time/1000, amount/1000)

		while (event_time >= max_time) {
			printf ("%.6f %d %.6f\n",
				rel_time/1000,
				num_samples,
				acum_size/granularity)

			min_time = max_time
			max_time = min_time + granularity
			rel_time += granularity
			acum_size = 0
			num_samples = 0
		}

		acum_size += amount
		num_samples++
	}
	END	{
		printf ("%.6f %d %.6f\n",
			rel_time/1000,
			num_samples,
			acum_size/granularity)
	}	
	'
}


################################################################################
# Parsing of command line options...
#
test "x$PATTERN" == "x" && echo "$0 needs an event and a trace file." && exit 1
test "x$FILE"    == "x" && echo "$0 needs an event and a trace file." && exit 1
test ! -f "$FILE"    && echo "'$FILE' file can not be read." && exit 1

if test -z "$GRAN" ; then
	GRAN=$DEF_GRAN
fi

avg $PATTERN $FILE $GRAN




