#! /bin/sh
#

event () {
	EV_STR="s/$1//"
	DESC_STR="s/[\'\"].*[\'\"]//"

	grep " $1 " $2 | sed -e "s/$1//" | sed -e "s/[\'\"].*[\'\"]//" | sort | \
		awk '
		NR == 1 {
			rel_time	= 0
			last_time	= $1
		}
			{
			curr_time	= $1
			val		= $2
			rel_time	+= (curr_time - last_time)
		
			printf ("%f %f\n", rel_time, val)
		
			last_time = curr_time
		}'
}

################################################################################

load_rtplog() {
	cat $1 |
	awk '
		{
		curr_time = $1
		curr_seq  = $18

		if (NR > 1 && curr_seq != 0)
			printf ("%.6f %d %d %d\n",		\
				curr_time - base_time,		\
				curr_seq - (prev_seq + 1),	\
				prev_seq, curr_seq)

		prev_seq  = curr_seq
	}
		
	NR == 1	{
		curr_seq  = $18
		prev_seq  = prev_seq
		curr_time = $1
		base_time = curr_time
	}
	'
}

load_event () {
	event $1 $2 |
	awk '
		{
		curr_time = $1
		curr_seq  = $2

		if (NR > 1 && curr_seq != 0)
			printf ("%.6f %d %d %d\n",		\
				curr_time - base_time,		\
				curr_seq - (prev_seq + 1),	\
				prev_seq, curr_seq)

		prev_seq  = curr_seq
	}
		
	NR == 1	{
		curr_seq  = $2
		prev_seq  = prev_seq
		curr_time = $1
		base_time = curr_time
	}
	'
}

################################################################################
# Parsing of command line options...
#
usage="\
Usage: $0 [-r|--rtp] \
[-e|--event <event>]"

while test $# -gt 0; do
	case $1 in
  		-r|--rtp)
      			RTP="true"
			shift
			;;
  		-e|--event)
      			EVENT=$2
			shift
			;;
		-h|--help)
			echo "${usage}" 1>&2
			exit 0
			;;
      		*)
			if test -f $1 ; then
       				FILE=$1
       				shift
			else
				echo "Unrecognized command '$1'"
				echo "${usage}" 1>&2
				exit 1
			fi
       		;;
    	esac
	shift
done   

test "x$FILE" != "x" || (echo "$0 needs a filename." ; exit 1)

if test "$RTP" == "true"; then
	load_rtplog $FILE
elif test "x$EVENT" != "x"; then
	load_event $EVENT $FILE
else
	echo "${usage}" 1>&2
	exit 1
fi

